@@ -13,12 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-import mock
-import six.moves.urllib.parse as urlparse
 import unittest
 
-from oslo_serialization import jsonutils
+import mock
 from requests import exceptions as requests_exceptions
+from requests import models
+import six.moves.urllib.parse as urlparse
 
 from vmware_nsxlib.tests.unit.v3 import mocks
 from vmware_nsxlib.tests.unit.v3 import nsxlib_testcase
@@ -36,6 +36,13 @@ def _validate_conn_down(*args, **kwargs):
     raise requests_exceptions.ConnectionError()
 
 
+def get_sess_create_resp():
+    sess_create_response = models.Response()
+    sess_create_response.status_code = 200
+    sess_create_response.headers = {'Set-Cookie': 'JSESSIONID=abc;'}
+    return sess_create_response
+
+
 class RequestsHTTPProviderTestCase(unittest.TestCase):
 
     def test_new_connection(self):
@@ -50,15 +57,18 @@ class RequestsHTTPProviderTestCase(unittest.TestCase):
         mock_api.nsxlib_config.conn_idle_timeout = 39
         mock_api.nsxlib_config.client_cert_provider = None
         provider = cluster.NSXRequestsHTTPProvider()
-        session = provider.new_connection(
-            mock_api, cluster.Provider('9.8.7.6', 'https://9.8.7.6',
-                                       'nsxuser', 'nsxpassword', None))
+        with mock.patch.object(cluster.TimeoutSession, 'request',
+                               return_value=get_sess_create_resp()):
+            session = provider.new_connection(
+                mock_api, cluster.Provider('9.8.7.6', 'https://9.8.7.6',
+                                           'nsxuser', 'nsxpassword', None))
 
-        self.assertEqual(session.auth, ('nsxuser', 'nsxpassword'))
-        self.assertEqual(session.verify, False)
-        self.assertIsNone(session.cert)
-        self.assertEqual(session.adapters['https://'].max_retries.total, 100)
-        self.assertEqual(session.timeout, 99)
+            self.assertEqual(('nsxuser', 'nsxpassword'), session.auth)
+            self.assertFalse(session.verify)
+            self.assertIsNone(session.cert)
+            self.assertEqual(100,
+                             session.adapters['https://'].max_retries.total)
+            self.assertEqual(99, session.timeout)
 
     def test_new_connection_with_client_auth(self):
         mock_api = mock.Mock()
@@ -72,29 +82,34 @@ class RequestsHTTPProviderTestCase(unittest.TestCase):
             '/etc/cert.pem')
         mock_api.nsxlib_config.client_cert_provider = cert_provider_inst
         provider = cluster.NSXRequestsHTTPProvider()
-        session = provider.new_connection(
-            mock_api, cluster.Provider('9.8.7.6', 'https://9.8.7.6',
-                                       None, None, None))
+        with mock.patch.object(cluster.TimeoutSession, 'request',
+                               return_value=get_sess_create_resp()):
+            session = provider.new_connection(
+                mock_api, cluster.Provider('9.8.7.6', 'https://9.8.7.6',
+                                           None, None, None))
 
-        self.assertEqual(session.auth, None)
-        self.assertEqual(session.verify, False)
-        self.assertEqual(session.cert_provider, cert_provider_inst)
-        self.assertEqual(session.timeout, 99)
+            self.assertIsNone(session.auth)
+            self.assertFalse(session.verify)
+            self.assertEqual(cert_provider_inst, session.cert_provider)
+            self.assertEqual(99, session.timeout)
 
     def test_validate_connection(self):
-        self.skipTest("Revist")
         mock_conn = mocks.MockRequestSessionApi()
+        mock_conn.default_headers = {}
         mock_ep = mock.Mock()
         mock_ep.provider.url = 'https://1.2.3.4'
+        mock_cluster = mock.Mock()
+        mock_cluster.nsxlib_config = mock.Mock()
+        mock_cluster.nsxlib_config.url_base = 'abc'
+        mock_cluster.nsxlib_config.keepalive_section = 'transport-zones'
         provider = cluster.NSXRequestsHTTPProvider()
         self.assertRaises(nsxlib_exc.ResourceNotFound,
                           provider.validate_connection,
-                          mock.Mock(), mock_ep, mock_conn)
+                          mock_cluster, mock_ep, mock_conn)
 
-        mock_conn.post('api/v1/transport-zones',
-                       data=jsonutils.dumps({'id': 'dummy-tz'}),
-                       headers=client.JSONRESTClient._DEFAULT_HEADERS)
-        provider.validate_connection(mock.Mock(), mock_ep, mock_conn)
+        with mock.patch.object(client.JSONRESTClient, "get",
+                               return_value={'result_count': 1}):
+            provider.validate_connection(mock_cluster, mock_ep, mock_conn)
 
 
 class NsxV3ClusteredAPITestCase(nsxlib_testcase.NsxClientTestCase):
@@ -162,7 +177,7 @@ class ClusteredAPITestCase(nsxlib_testcase.NsxClientTestCase):
         conf_managers = ['8.9.10.11', '9.10.11.12']
         api = self.new_mocked_cluster(conf_managers, validate_fn)
 
-        self.assertEqual(api.health, expected_health)
+        self.assertEqual(expected_health, api.health)
 
     def test_orange_health(self):
 
@@ -182,7 +197,7 @@ class ClusteredAPITestCase(nsxlib_testcase.NsxClientTestCase):
         conf_managers = ['8.9.10.11', '9.10.11.12', '10.11.12.13']
         api = self.new_mocked_cluster(conf_managers, _validate_conn_down)
 
-        self.assertEqual(len(api.endpoints), 3)
+        self.assertEqual(3, len(api.endpoints))
         self.assertRaises(nsxlib_exc.ServiceClusterUnavailable,
                           api.get, 'api/v1/transport-zones')
 
@@ -231,6 +246,8 @@ class ClusteredAPITestCase(nsxlib_testcase.NsxClientTestCase):
         self.assertEqual(_get_schedule(4), [eps[0], eps[2], eps[0], eps[2]])
 
     def test_reinitialize_cluster(self):
-        api = self.mock_nsx_clustered_api()
-        # just make sure this api is defined, and does not crash
-        api._reinit_cluster()
+        with mock.patch.object(cluster.TimeoutSession, 'request',
+                               return_value=get_sess_create_resp()):
+            api = self.mock_nsx_clustered_api()
+            # just make sure this api is defined, and does not crash
+            api._reinit_cluster()
