@@ -17,21 +17,24 @@ import abc
 import contextlib
 import copy
 import datetime
-import eventlet
+import inspect
 import itertools
 import logging
+import re
+
+import eventlet
+from eventlet import greenpool
+from eventlet import pools
 import OpenSSL
+from oslo_log import log
+from oslo_service import loopingcall
 import requests
+from requests import adapters
+from requests import exceptions as requests_exceptions
 import six
 import six.moves.urllib.parse as urlparse
 
-from eventlet import greenpool
-from eventlet import pools
-from oslo_log import log
-from oslo_service import loopingcall
-from requests import adapters
-from requests import exceptions as requests_exceptions
-from vmware_nsxlib._i18n import _, _LI, _LW
+from vmware_nsxlib._i18n import _
 from vmware_nsxlib.v3 import client as nsx_client
 from vmware_nsxlib.v3 import exceptions
 
@@ -102,39 +105,53 @@ class TimeoutSession(requests.Session):
     # wrapper timeouts at the session level
     # see: https://goo.gl/xNk7aM
     def request(self, *args, **kwargs):
+        def request_with_retry_on_ssl_error(*args, **kwargs):
+            try:
+                return super(TimeoutSession, self).request(*args, **kwargs)
+            except OpenSSL.SSL.Error:
+                # This can happen when connection tries to access certificate
+                # file it was opened with (renegotiation?)
+                # Proper way to solve this would be to pass in-memory cert
+                # to ssl C code.
+                # Retrying here works around the problem
+                return super(TimeoutSession, self).request(*args, **kwargs)
+
+        def get_cert_provider():
+            if inspect.isclass(self._cert_provider):
+                # If client provided certificate provider as a class,
+                # we spawn an instance here
+                return self._cert_provider()
+            return self._cert_provider
+
         if 'timeout' not in kwargs:
             kwargs['timeout'] = (self.timeout, self.read_timeout)
-
-        if not self._cert_provider:
+        if not self.cert_provider:
+            # No client certificate needed
             return super(TimeoutSession, self).request(*args, **kwargs)
 
         if self.cert is not None:
-            # connection should be open (unless server closed it),
-            # in which case cert is not needed
-            try:
-                return super(TimeoutSession, self).request(*args, **kwargs)
-            except OpenSSL.SSL.Error as e:
-                # This is most probably "client cert not found" error (this
-                # happens when server closed the connection and requests
-                # reopen it). Try reloading client cert.
-                LOG.warning(_LW("SSL error: %s, retrying..") % e)
-            except OSError as e:
-                # Lack of client cert file can come in form of OSError,
-                # in this case filename will appear in the error. Try
-                # reloading client cert.
-                if self._cert_provider.filename() not in str(e):
-                    raise e
-                # Don't expose cert file name to the logs
-                LOG.warning(_LW("Reloading client certificate.."))
+            # Recursive call - shouldn't happen
+            return request_with_retry_on_ssl_error(*args, **kwargs)
 
         # The following with statement allows for preparing certificate and
-        # private key file and dispose it once connections are spawned
+        # private key file and dispose it at the end of request
         # (since PK is sensitive information, immediate disposal is
-        # important). This is done of first request of the session or when
-        # above exceptions indicate cert is missing.
-        with self._cert_provider:
-            self.cert = self._cert_provider.filename()
-            ret = super(TimeoutSession, self).request(*args, **kwargs)
+        # important).
+        # It would be optimal to populate certificate once per connection,
+        # per request. Unfortunately requests library verifies cert file
+        # existence regardless of whether certificate is going to be used
+        # for this request.
+        # Optimal solution for this would be to expose certificate as variable
+        # and not as a file to the SSL library
+        with get_cert_provider() as provider:
+            self.cert = provider.filename()
+            try:
+                ret = request_with_retry_on_ssl_error(*args, **kwargs)
+            except Exception as e:
+                self.cert = None
+                raise e
+
+            self.cert = None
 
         return ret
 
@@ -145,16 +162,28 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
     using requests.Session() as the underlying connection.
     """
 
+    SESSION_CREATE_URL = '/api/session/create'
+    COOKIE_FIELD = 'Cookie'
+    SET_COOKIE_FIELD = 'Set-Cookie'
+    XSRF_TOKEN = 'X-XSRF-TOKEN'
+    JSESSIONID = 'JSESSIONID'
+
     @property
     def provider_id(self):
         return "%s-%s" % (requests.__title__, requests.__version__)
 
     def validate_connection(self, cluster_api, endpoint, conn):
-        client = nsx_client.NSX3Client(conn, url_prefix=endpoint.provider.url)
-        zones = client.get('transport-zones', silent=True)
-        if not zones or zones['result_count'] <= 0:
-            msg = _("No transport zones found "
-                    "for '%s'") % endpoint.provider.url
+        client = nsx_client.NSX3Client(
+            conn, url_prefix=endpoint.provider.url,
+            url_path_base=cluster_api.nsxlib_config.url_base,
+            default_headers=conn.default_headers)
+        keepalive_section = cluster_api.nsxlib_config.keepalive_section
+        result = client.get(keepalive_section, silent=True)
+        # If keeplive section returns a list, it is assumed to be non-empty
+        if not result or result.get('result_count', 1) <= 0:
+            msg = _("No %(section)s found "
+                    "for '%(url)s'") % {'section': keepalive_section,
+                                        'url': endpoint.provider.url}
             LOG.warning(msg)
             raise exceptions.ResourceNotFound(
                 manager=endpoint.provider.url, operation=msg)
@@ -184,11 +213,55 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
         session.mount('http://', adapter)
         session.mount('https://', adapter)
 
+        self.get_default_headers(session, provider,
+                                 config.allow_overwrite_header)
+
         return session
 
     def is_connection_exception(self, exception):
         return isinstance(exception, requests_exceptions.ConnectionError)
 
+    def get_default_headers(self, session, provider, allow_overwrite_header):
+        """Get the default headers that should be added to future requests"""
+        session.default_headers = {}
+
+        # Perform the initial session create and get the relevant jsessionid &
+        # X-XSRF-TOKEN for future requests
+        req_data = ''
+        if not session.cert_provider:
+            # With client certificate authentication, username and password
+            # may not be provided.
+            # If provided, backend treats these credentials as authentication
+            # and ignores client cert as principal identity indication.
+            req_data = 'j_username=%s&j_password=%s' % (provider.username,
+                                                        provider.password)
+        req_headers = {'Accept': 'application/json',
+                       'Content-Type': 'application/x-www-form-urlencoded'}
+        # Cannot use the certificate at this stage, because it is used for
+        # the certificate generation
+        resp = session.request('post', provider.url + self.SESSION_CREATE_URL,
+                               data=req_data, headers=req_headers)
+        if resp.status_code != 200:
+            LOG.error("Session create failed for endpoint %s", provider.url)
+            # this will later cause the endpoint to be Down
+        else:
+            for header_name in resp.headers:
+                if self.SET_COOKIE_FIELD.lower() == header_name.lower():
+                    m = re.match('%s=.*?\;' % self.JSESSIONID,
+                                 resp.headers[header_name])
+                    if m:
+                        session.default_headers[self.COOKIE_FIELD] = m.group()
+                if self.XSRF_TOKEN.lower() == header_name.lower():
+                    session.default_headers[self.XSRF_TOKEN] = resp.headers[
+                        header_name]
+            LOG.info("Session create succeeded for endpoint %(url)s with "
+                     "headers %(hdr)s",
+                     {'url': provider.url, 'hdr': session.default_headers})
+
+        # Add allow-overwrite if configured
+        if allow_overwrite_header:
+            session.default_headers['X-Allow-Overwrite'] = 'true'
+
 
 class ClusterHealth(object):
     """Indicator of overall cluster health.
@@ -246,6 +319,12 @@ class Endpoint(object):
         self._state = EndpointState.INITIALIZED
         self._last_updated = datetime.datetime.now()
 
+    def regenerate_pool(self):
+        self.pool = pools.Pool(min_size=self.pool.min_size,
+                               max_size=self.pool.max_size,
+                               order_as_stack=True,
+                               create=self.pool.create)
+
     @property
     def last_updated(self):
         return self._last_updated
@@ -256,8 +335,8 @@ class Endpoint(object):
 
     def set_state(self, state):
         if self.state != state:
-            LOG.info(_LI("Endpoint '%(ep)s' changing from state"
-                         " '%(old)s' to '%(new)s'"),
+            LOG.info("Endpoint '%(ep)s' changing from state"
+                     " '%(old)s' to '%(new)s'",
                      {'ep': self.provider,
                       'old': self.state,
                       'new': state})
@@ -404,10 +483,22 @@ class ClusteredAPI(object):
             with endpoint.pool.item() as conn:
                 self._http_provider.validate_connection(self, endpoint, conn)
                 endpoint.set_state(EndpointState.UP)
+        except exceptions.ClientCertificateNotTrusted:
+            LOG.warning("Failed to validate API cluster endpoint "
+                        "'%(ep)s' due to untrusted client certificate",
+                        {'ep': endpoint})
+            # regenerate connection pool based on new certificate
+            endpoint.regenerate_pool()
+        except exceptions.BadXSRFToken:
+            LOG.warning("Failed to validate API cluster endpoint "
+                        "'%(ep)s' due to expired XSRF token",
+                        {'ep': endpoint})
+            # regenerate connection pool based on token
+            endpoint.regenerate_pool()
         except Exception as e:
             endpoint.set_state(EndpointState.DOWN)
-            LOG.warning(_LW("Failed to validate API cluster endpoint "
-                            "'%(ep)s' due to: %(err)s"),
+            LOG.warning("Failed to validate API cluster endpoint "
+                        "'%(ep)s' due to: %(err)s",
                         {'ep': endpoint, 'err': e})
 
     def _select_endpoint(self):
@@ -448,8 +539,8 @@ class ClusteredAPI(object):
                 cluster_id=self.cluster_id)
 
         if endpoint.pool.free() == 0:
-            LOG.info(_LI("API endpoint %(ep)s at connection "
-                         "capacity %(max)s and has %(waiting)s waiting"),
+            LOG.info("API endpoint %(ep)s at connection "
+                     "capacity %(max)s and has %(waiting)s waiting",
                      {'ep': endpoint,
                       'max': endpoint.pool.max_size,
                       'waiting': endpoint.pool.waiting()})
@@ -477,6 +568,11 @@ class ClusteredAPI(object):
             try:
                 LOG.debug("API cluster proxy %s %s to %s",
                           proxy_for.upper(), uri, url)
+                # Add the connection default headers
+                if conn.default_headers:
+                    kwargs['headers'] = kwargs.get('headers', {})
+                    kwargs['headers'].update(conn.default_headers)
+
                 # call the actual connection method to do the
                 # http request/response over the wire
                 response = do_request(url, *args, **kwargs)
@@ -484,7 +580,7 @@ class ClusteredAPI(object):
 
                 return response
             except Exception as e:
-                LOG.warning(_LW("Request failed due to: %s"), e)
+                LOG.warning("Request failed due to: %s", e)
                 if not self._http_provider.is_connection_exception(e):
                     # only trap and retry connection errors
                     raise e
@@ -529,8 +625,8 @@ class NSXClusteredAPI(ClusteredAPI):
             provider_index += 1
             conf_url = _schemed_url(conf_url)
             if conf_url in urls:
-                LOG.warning(_LW("'%s' already defined in configuration file. "
-                                "Skipping."), urlparse.urlunparse(conf_url))
+                LOG.warning("'%s' already defined in configuration file. "
+                            "Skipping.", urlparse.urlunparse(conf_url))
                 continue
             urls.append(conf_url)
             providers.append(
