@@ -63,6 +63,10 @@ class CertificateError(NsxLibException):
     message = _("Certificate error: %(msg)s")
 
 
+class NsxLibInvalidInput(NsxLibException):
+    message = _("Invalid input for operation: %(error_message)s.")
+
+
 class ManagerError(NsxLibException):
     message = _("Unexpected error from backend manager (%(manager)s) "
                 "for %(operation)s %(details)s")
@@ -83,6 +87,11 @@ class ResourceNotFound(ManagerError):
                 "%(operation)s")
 
 
+class BackendResourceNotFound(ResourceNotFound):
+    message = _("%(details)s On backend (%(manager)s) with Operation: "
+                "%(operation)s")
+
+
 class InvalidInput(ManagerError):
     message = _("%(operation)s failed: Invalid input %(arg_val)s "
                 "for %(arg_name)s")
@@ -92,10 +101,18 @@ class StaleRevision(ManagerError):
     pass
 
 
+class TooManyRequests(ManagerError):
+    pass
+
+
 class ClientCertificateNotTrusted(ManagerError):
     message = _("Certificate not trusted")
 
 
+class BadXSRFToken(ManagerError):
+    message = _("Bad or expired XSRF token")
+
+
 class ServiceClusterUnavailable(ManagerError):
     message = _("Service cluster: '%(cluster_id)s' is unavailable. Please, "
                 "check NSX setup and/or configuration")
