@@ -28,6 +28,9 @@ ATTACHMENT_MDPROXY = "METADATA_PROXY"
 
 VIF_RESOURCE_TYPE = "VifAttachmentContext"
 
+VIF_TYPE_PARENT = "PARENT"
+VIF_TYPE_CHILD = "CHILD"
+
 ALLOCATE_ADDRESS_NONE = "None"
 
 # NSXv3 L2 Gateway constants
@@ -39,6 +42,7 @@ ROUTER_TYPE_TIER1 = "TIER1"
 
 LROUTERPORT_UPLINK = "LogicalRouterUplinkPort"
 LROUTERPORT_DOWNLINK = "LogicalRouterDownLinkPort"
+LROUTERPORT_CENTRALIZED = "LogicalRouterCentralizedServicePort"
 LROUTERPORT_LINKONTIER0 = "LogicalRouterLinkPortOnTIER0"
 LROUTERPORT_LINKONTIER1 = "LogicalRouterLinkPortOnTIER1"
 
@@ -46,13 +50,16 @@ LROUTERPORT_LINKONTIER1 = "LogicalRouterLinkPortOnTIER1"
 SERVICE_DHCP = "dhcp"
 
 # NSX-V3 Distributed Firewall constants
+IP_SET = 'IPSet'
 NSGROUP = 'NSGroup'
+NSGROUP_COMPLEX_EXP = 'NSGroupComplexExpression'
 NSGROUP_SIMPLE_EXP = 'NSGroupSimpleExpression'
 NSGROUP_TAG_EXP = 'NSGroupTagExpression'
 EXCLUDE_PORT = 'Exclude-Port'
 
 # Firewall rule position
 FW_INSERT_BEFORE = 'insert_before'
+FW_INSERT_AFTER = 'insert_after'
 FW_INSERT_BOTTOM = 'insert_bottom'
 FW_INSERT_TOP = 'insert_top'
 
@@ -107,7 +114,29 @@ INGRESS_SHAPING = 'IngressRateShaper'
 ERR_CODE_OBJECT_NOT_FOUND = 202
 ERR_CODE_IPAM_POOL_EXHAUSTED = 5109
 ERR_CODE_IPAM_SPECIFIC_IP = 5123
+ERR_CODE_IPAM_IP_ALLOCATED = 5141
 ERR_CODE_IPAM_IP_NOT_IN_POOL = 5110
 ERR_CODE_IPAM_RANGE_MODIFY = 5602
 ERR_CODE_IPAM_RANGE_DELETE = 5015
 ERR_CODE_IPAM_RANGE_SHRUNK = 5016
+
+# backend versions
+NSX_VERSION_1_1_0 = '1.1.0'
+NSX_VERSION_2_0_0 = '2.0.0'
+NSX_VERSION_2_1_0 = '2.1.0'
+NSX_VERSION_2_2_0 = '2.2.0'
+NSX_VERSION_3_0_0 = '3.0.0'
+
+# Features available depending on the backend version
+FEATURE_MAC_LEARNING = 'MAC Learning'
+FEATURE_DYNAMIC_CRITERIA = 'Dynamic criteria'
+FEATURE_EXCLUDE_PORT_BY_TAG = 'Exclude Port by Tag'
+FEATURE_ROUTER_FIREWALL = 'Router Firewall'
+FEATURE_LOAD_BALANCER = 'Load Balancer'
+FEATURE_DHCP_RELAY = 'DHCP Relay'
+FEATURE_NSX_POLICY = 'NSX Policy'
+FEATURE_VLAN_ROUTER_INTERFACE = 'VLAN Router Interface'
+FEATURE_RATE_LIMIT = 'Requests Rate Limit'
+FEATURE_IPSEC_VPN = 'IPSec VPN'
+FEATURE_ON_BEHALF_OF = 'On Behalf Of'
+FEATURE_TRUNK_VLAN = 'Trunk Vlan'
