@@ -13,194 +13,77 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-import abc
-import collections
 import netaddr
-import six
+
+from oslo_log import log
+from oslo_log import versionutils
+import requests
 
 from vmware_nsxlib._i18n import _
+from vmware_nsxlib.v3 import core_resources
 from vmware_nsxlib.v3 import exceptions
 from vmware_nsxlib.v3 import nsx_constants
 from vmware_nsxlib.v3 import utils
 
+LOG = log.getLogger(__name__)
 
-SwitchingProfileTypeId = collections.namedtuple(
-    'SwitchingProfileTypeId', 'profile_type, profile_id')
+# TODO(asarfaty): keeping this for backwards compatibility.
+# core_resources.SwitchingProfileTypeId and
+# core_resources.PacketAddressClassifier should be used.
+# This code will be removed in the future.
+SwitchingProfileTypeId = core_resources.SwitchingProfileTypeId
+PacketAddressClassifier = core_resources.PacketAddressClassifier
 
 
-PacketAddressClassifier = collections.namedtuple(
-    'PacketAddressClassifier', 'ip_address, mac_address, vlan')
+class SwitchingProfileTypes(core_resources.SwitchingProfileTypes):
+    # TODO(asarfaty): keeping this for backwards compatibility.
+    # This code will be removed in the future.
+    def __init__(self):
+        versionutils.report_deprecated_feature(
+            LOG,
+            'resources.SwitchingProfileTypes is deprecated. '
+            'Please use core_resources.SwitchingProfileTypes instead.')
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractRESTResource(object):
+class WhiteListAddressTypes(core_resources.WhiteListAddressTypes):
+    # TODO(asarfaty): keeping this for backwards compatibility.
+    # This code will be removed in the future.
+    def __init__(self):
+        versionutils.report_deprecated_feature(
+            LOG,
+            'resources.WhiteListAddressTypes is deprecated. '
+            'Please use core_resources.WhiteListAddressTypes instead.')
 
+
+class SwitchingProfile(core_resources.NsxLibSwitchingProfile):
+    # TODO(asarfaty): keeping this for backwards compatibility.
+    # This code will be removed in the future.
     def __init__(self, rest_client, *args, **kwargs):
-        self._client = rest_client.new_client_for(self.uri_segment)
-
-    @abc.abstractproperty
-    def uri_segment(self):
-        pass
-
-    def list(self):
-        return self._client.list()
-
-    def get(self, uuid):
-        return self._client.get(uuid)
-
-    def delete(self, uuid):
-        return self._client.delete(uuid)
-
-    @abc.abstractmethod
-    def create(self, *args, **kwargs):
-        pass
-
-    @abc.abstractmethod
-    def update(self, uuid, *args, **kwargs):
-        pass
-
-    def find_by_display_name(self, display_name):
-        found = []
-        for resource in self.list()['results']:
-            if resource['display_name'] == display_name:
-                found.append(resource)
-        return found
+        versionutils.report_deprecated_feature(
+            LOG,
+            'resources.SwitchingProfile is deprecated. '
+            'Please use core_resources.NsxLibSwitchingProfile instead.')
+        super(SwitchingProfile, self).__init__(rest_client)
 
 
-class SwitchingProfileTypes(object):
-    IP_DISCOVERY = 'IpDiscoverySwitchingProfile'
-    MAC_LEARNING = 'MacManagementSwitchingProfile'
-    PORT_MIRRORING = 'PortMirroringSwitchingProfile'
-    QOS = 'QosSwitchingProfile'
-    SPOOF_GUARD = 'SpoofGuardSwitchingProfile'
-    SWITCH_SECURITY = 'SwitchSecuritySwitchingProfile'
-
-
-class WhiteListAddressTypes(object):
-    PORT = 'LPORT_BINDINGS'
-    SWITCH = 'LSWITCH_BINDINGS'
-
-
-class SwitchingProfile(AbstractRESTResource):
-
-    @property
-    def uri_segment(self):
-        return 'switching-profiles'
-
-    def list(self):
-        return self._client.url_get('?include_system_owned=True')
-
-    def create(self, profile_type, display_name=None,
-               description=None, **api_args):
-        body = {
-            'resource_type': profile_type,
-            'display_name': display_name or '',
-            'description': description or ''
-        }
-        body.update(api_args)
-
-        return self._client.create(body=body)
-
-    def update(self, uuid, profile_type, **api_args):
-        body = {
-            'resource_type': profile_type
-        }
-        body.update(api_args)
-
-        return self._client.update(uuid, body=body)
-
-    def create_spoofguard_profile(self, display_name,
-                                  description,
-                                  whitelist_ports=False,
-                                  whitelist_switches=False,
-                                  tags=None):
-        whitelist_providers = []
-        if whitelist_ports:
-            whitelist_providers.append(WhiteListAddressTypes.PORT)
-        if whitelist_switches:
-            whitelist_providers.append(WhiteListAddressTypes.SWITCH)
-
-        return self.create(SwitchingProfileTypes.SPOOF_GUARD,
-                           display_name=display_name,
-                           description=description,
-                           white_list_providers=whitelist_providers,
-                           tags=tags or [])
-
-    def create_dhcp_profile(self, display_name,
-                            description, tags=None):
-        dhcp_filter = {
-            'client_block_enabled': True,
-            'server_block_enabled': False
-        }
-        rate_limits = {
-            'enabled': False,
-            'rx_broadcast': 0,
-            'tx_broadcast': 0,
-            'rx_multicast': 0,
-            'tx_multicast': 0
-        }
-        bpdu_filter = {
-            'enabled': True,
-            'white_list': []
-        }
-        return self.create(SwitchingProfileTypes.SWITCH_SECURITY,
-                           display_name=display_name,
-                           description=description,
-                           tags=tags or [],
-                           dhcp_filter=dhcp_filter,
-                           rate_limits=rate_limits,
-                           bpdu_filter=bpdu_filter,
-                           block_non_ip_traffic=True)
-
-    def create_mac_learning_profile(self, display_name,
-                                    description, tags=None):
-        mac_learning = {
-            'enabled': True,
-        }
-        return self.create(SwitchingProfileTypes.MAC_LEARNING,
-                           display_name=display_name,
-                           description=description,
-                           tags=tags or [],
-                           mac_learning=mac_learning,
-                           mac_change_allowed=True)
-
-    def create_port_mirror_profile(self, display_name, description,
-                                   direction, destinations, tags=None):
-        return self.create(SwitchingProfileTypes.PORT_MIRRORING,
-                           display_name=display_name,
-                           description=description,
-                           tags=tags or [],
-                           direction=direction,
-                           destinations=destinations)
-
-    @classmethod
-    def build_switch_profile_ids(cls, client, *profiles):
-        ids = []
-        for profile in profiles:
-            if isinstance(profile, str):
-                profile = client.get(profile)
-            if not isinstance(profile, SwitchingProfileTypeId):
-                profile = SwitchingProfileTypeId(
-                    profile.get('key', profile.get('resource_type')),
-                    profile.get('value', profile.get('id')))
-            ids.append(profile)
-        return ids
-
-
-class LogicalPort(AbstractRESTResource):
+class LogicalPort(utils.NsxLibApiBase):
 
     @property
     def uri_segment(self):
         return 'logical-ports'
 
+    @property
+    def resource_type(self):
+        return 'LogicalPort'
+
     def _build_body_attrs(
             self, display_name=None,
             admin_state=True, tags=None,
             address_bindings=None,
             switch_profile_ids=None,
-            attachment=None):
+            attachment=None,
+            description=None):
         tags = tags or []
-        address_bindings = address_bindings or []
         switch_profile_ids = switch_profile_ids or []
         body = {}
         if tags:
@@ -225,8 +108,7 @@ class LogicalPort(AbstractRESTResource):
                     address_classifier['vlan'] = int(binding.vlan)
                 bindings.append(address_classifier)
             body['address_bindings'] = bindings
-        elif address_bindings == []:
-            # explicitly clear out address bindings
+        elif address_bindings is not None:
             body['address_bindings'] = []
 
         if switch_profile_ids:
@@ -242,6 +124,9 @@ class LogicalPort(AbstractRESTResource):
         if attachment is not False:
             body['attachment'] = attachment
 
+        if description is not None:
+            body['description'] = description
+
         return body
 
     def _prepare_attachment(self, attachment_type, vif_uuid,
@@ -265,22 +150,14 @@ class LogicalPort(AbstractRESTResource):
         else:
             return False  # no attachment change
 
-    def _build_address_bindings(self, address_bindings):
-        addr_bindings = []
-        for binding in address_bindings:
-            addr_bindings.append(PacketAddressClassifier(
-                binding.get('ip_address'), binding.get('mac_address'),
-                binding.get('vlan')))
-        return addr_bindings
-
     def create(self, lswitch_id, vif_uuid, tags=None,
                attachment_type=nsx_constants.ATTACHMENT_VIF,
                admin_state=True, name=None, address_bindings=None,
                parent_vif_id=None, traffic_tag=None,
                switch_profile_ids=None, vif_type=None, app_id=None,
-               allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE):
+               allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE,
+               description=None):
         tags = tags or []
-
         body = {'logical_switch_id': lswitch_id}
         # NOTE(arosen): If parent_vif_id is specified we need to use
         # CIF attachment type.
@@ -293,18 +170,12 @@ class LogicalPort(AbstractRESTResource):
             admin_state=admin_state, tags=tags,
             address_bindings=address_bindings,
             switch_profile_ids=switch_profile_ids,
-            attachment=attachment))
-        return self._client.create(body=body)
+            attachment=attachment,
+            description=description))
+        return self.client.create(self.get_path(), body=body)
 
     def delete(self, lport_id):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def _do_delete():
-            return self._client.url_delete('%s?detach=true' % lport_id)
-
-        return _do_delete()
+        self._delete_with_retry('%s?detach=true' % lport_id)
 
     def update(self, lport_id, vif_uuid,
                name=None, admin_state=None,
@@ -313,88 +184,55 @@ class LogicalPort(AbstractRESTResource):
                attachment_type=nsx_constants.ATTACHMENT_VIF,
                parent_vif_id=None, traffic_tag=None,
                vif_type=None, app_id=None,
-               allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def do_update():
-            lport = self.get(lport_id)
-            tags = lport.get('tags', [])
-            if tags_update:
-                tags = utils.update_v3_tags(tags, tags_update)
-            # Assign outer function argument to a local scope
-            addr_bindings = address_bindings
-            if addr_bindings is None:
-                addr_bindings = self._build_address_bindings(
-                    lport.get('address_bindings'))
-            attachment = self._prepare_attachment(attachment_type, vif_uuid,
-                                                  allocate_addresses, vif_type,
-                                                  parent_vif_id, traffic_tag,
-                                                  app_id)
-            lport.update(self._build_body_attrs(
-                display_name=name,
-                admin_state=admin_state, tags=tags,
-                address_bindings=addr_bindings,
-                switch_profile_ids=switch_profile_ids,
-                attachment=attachment))
-
-            # If revision_id of the payload that we send is older than what
-            # NSX has, we will get a 412: Precondition Failed.
-            # In that case we need to re-fetch, patch the response and send
-            # it again with the new revision_id
-            return self._client.update(lport_id, body=lport)
-        return do_update()
-
-
-class LogicalRouter(AbstractRESTResource):
-
-    @property
-    def uri_segment(self):
-        return 'logical-routers'
-
-    def create(self, display_name, tags, edge_cluster_uuid=None, tier_0=False,
+               allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE,
                description=None):
-        # TODO(salv-orlando): If possible do not manage edge clusters
-        # in the main plugin logic.
-        router_type = (nsx_constants.ROUTER_TYPE_TIER0 if tier_0 else
-                       nsx_constants.ROUTER_TYPE_TIER1)
-        body = {'display_name': display_name,
-                'router_type': router_type,
-                'tags': tags}
-        if edge_cluster_uuid:
-            body['edge_cluster_id'] = edge_cluster_uuid
-        if description:
-            body['description'] = description
-        return self._client.create(body=body)
+        attachment = self._prepare_attachment(attachment_type, vif_uuid,
+                                              allocate_addresses, vif_type,
+                                              parent_vif_id, traffic_tag,
+                                              app_id)
+        lport = {}
+        if tags_update is not None:
+            lport['tags_update'] = tags_update
+        lport.update(self._build_body_attrs(
+            display_name=name,
+            admin_state=admin_state,
+            address_bindings=address_bindings,
+            switch_profile_ids=switch_profile_ids,
+            attachment=attachment,
+            description=description))
 
-    def delete(self, lrouter_id):
-        return self._client.url_delete(lrouter_id)
+        return self._update_resource(
+            self.get_path(lport_id), lport, retry=True)
 
-    def update(self, lrouter_id, *args, **kwargs):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def _do_update():
-            lrouter = self.get(lrouter_id)
-            for k in kwargs:
-                lrouter[k] = kwargs[k]
-            # If revision_id of the payload that we send is older than what
-            # NSX has, we will get a 412: Precondition Failed.
-            # In that case we need to re-fetch, patch the response and send
-            # it again with the new revision_id
-            return self._client.update(lrouter_id, body=lrouter)
-
-        return _do_update()
+    def get_by_attachment(self, attachment_type, attachment_id):
+        """Return all logical port matching the attachment type and Id"""
+        url_suffix = ('?attachment_type=%s&attachment_id=%s' %
+                      (attachment_type, attachment_id))
+        return self.client.get(self.get_path(url_suffix))
 
 
-class LogicalRouterPort(AbstractRESTResource):
+class LogicalRouter(core_resources.NsxLibLogicalRouter):
+    # TODO(asarfaty): keeping this for backwards compatibility.
+    # This code will be removed in the future.
+    def __init__(self, rest_client, *args, **kwargs):
+        versionutils.report_deprecated_feature(
+            LOG,
+            'resources.LogicalRouter is deprecated. '
+            'Please use core_resources.NsxLibLogicalRouter instead.')
+        super(LogicalRouter, self).__init__(rest_client)
+
+
+class LogicalRouterPort(utils.NsxLibApiBase):
 
     @property
     def uri_segment(self):
         return 'logical-router-ports'
 
+    @staticmethod
+    def _get_relay_binding(relay_service_uuid):
+        return {'service_id': {'target_type': 'LogicalService',
+                               'target_id': relay_service_uuid}}
+
     def create(self, logical_router_id,
                display_name,
                tags,
@@ -402,7 +240,8 @@ class LogicalRouterPort(AbstractRESTResource):
                logical_port_id,
                address_groups,
                edge_cluster_member_index=None,
-               urpf_mode=None):
+               urpf_mode=None,
+               relay_service_uuid=None):
         body = {'display_name': display_name,
                 'resource_type': resource_type,
                 'logical_router_id': logical_router_id,
@@ -410,7 +249,8 @@ class LogicalRouterPort(AbstractRESTResource):
         if address_groups:
             body['subnets'] = address_groups
         if resource_type in [nsx_constants.LROUTERPORT_UPLINK,
-                             nsx_constants.LROUTERPORT_DOWNLINK]:
+                             nsx_constants.LROUTERPORT_DOWNLINK,
+                             nsx_constants.LROUTERPORT_CENTRALIZED]:
             body['linked_logical_switch_port_id'] = {
                 'target_id': logical_port_id}
         elif resource_type == nsx_constants.LROUTERPORT_LINKONTIER1:
@@ -422,39 +262,51 @@ class LogicalRouterPort(AbstractRESTResource):
             body['edge_cluster_member_index'] = edge_cluster_member_index
         if urpf_mode:
             body['urpf_mode'] = urpf_mode
+        if relay_service_uuid:
+            if (self.nsxlib and
+                self.nsxlib.feature_supported(
+                    nsx_constants.FEATURE_DHCP_RELAY)):
+                body['service_bindings'] = [self._get_relay_binding(
+                    relay_service_uuid)]
+            else:
+                LOG.error("Ignoring relay_service_uuid for router %s port: "
+                          "This feature is not supported.", logical_router_id)
 
-        return self._client.create(body=body)
+        return self.client.create(self.get_path(), body=body)
 
     def update(self, logical_port_id, **kwargs):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def _do_update():
-            logical_router_port = self.get(logical_port_id)
-            for k in kwargs:
-                logical_router_port[k] = kwargs[k]
-            # If revision_id of the payload that we send is older than what
-            # NSX has, we will get a 412: Precondition Failed.
-            # In that case we need to re-fetch, patch the response and send
-            # it again with the new revision_id
-            return self._client.update(logical_port_id,
-                                       body=logical_router_port)
-        return _do_update()
+        logical_router_port = {}
+        # special treatment for updating/removing the relay service
+        if 'relay_service_uuid' in kwargs:
+            if kwargs['relay_service_uuid']:
+                if (self.nsxlib and
+                    self.nsxlib.feature_supported(
+                        nsx_constants.FEATURE_DHCP_RELAY)):
+                    logical_router_port['service_bindings'] = [
+                        self._get_relay_binding(
+                            kwargs['relay_service_uuid'])]
+                else:
+                    LOG.error("Ignoring relay_service_uuid for router "
+                              "port %s: This feature is not supported.",
+                              logical_port_id)
+            else:
+                # delete the current one
+                if 'service_bindings' in logical_router_port:
+                    logical_router_port['service_bindings'] = []
+            del kwargs['relay_service_uuid']
+
+        for k in kwargs:
+            logical_router_port[k] = kwargs[k]
+
+        return self._update_resource(
+            self.get_path(logical_port_id), logical_router_port, retry=True)
 
     def delete(self, logical_port_id):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def _do_delete():
-            return self._client.url_delete(logical_port_id)
-
-        return _do_delete()
+        self._delete_with_retry(logical_port_id)
 
     def get_by_lswitch_id(self, logical_switch_id):
         resource = '?logical_switch_id=%s' % logical_switch_id
-        router_ports = self._client.url_get(resource)
+        router_ports = self.client.url_get(self.get_path(resource))
         result_count = int(router_ports.get('result_count', "0"))
         if result_count >= 2:
             raise exceptions.ManagerError(
@@ -466,7 +318,7 @@ class LogicalRouterPort(AbstractRESTResource):
             err_msg = (_("Logical router link port not found on logical "
                          "switch %s") % logical_switch_id)
             raise exceptions.ResourceNotFound(
-                manager=self._client.nsx_api_managers,
+                manager=self.client.nsx_api_managers,
                 operation=err_msg)
 
     def update_by_lswitch_id(self, logical_router_id, ls_id, **payload):
@@ -479,7 +331,7 @@ class LogicalRouterPort(AbstractRESTResource):
 
     def get_by_router_id(self, logical_router_id):
         resource = '?logical_router_id=%s' % logical_router_id
-        logical_router_ports = self._client.url_get(resource)
+        logical_router_ports = self.client.url_get(self.get_path(resource))
         return logical_router_ports['results']
 
     def get_tier1_link_port(self, logical_router_id):
@@ -488,37 +340,33 @@ class LogicalRouterPort(AbstractRESTResource):
             if port['resource_type'] == nsx_constants.LROUTERPORT_LINKONTIER1:
                 return port
         raise exceptions.ResourceNotFound(
-            manager=self._client.nsx_api_managers,
+            manager=self.client.nsx_api_managers,
             operation="get router link port")
 
 
-class MetaDataProxy(AbstractRESTResource):
-
-    @property
-    def uri_segment(self):
-        return 'md-proxies'
-
-    def create(self, *args, **kwargs):
-        pass
-
-    def update(self, uuid, *args, **kwargs):
-        pass
+class MetaDataProxy(core_resources.NsxLibMetadataProxy):
+    # TODO(asarfaty): keeping this for backwards compatibility.
+    # This code will be removed in the future.
+    def __init__(self, rest_client, *args, **kwargs):
+        versionutils.report_deprecated_feature(
+            LOG,
+            'resources.MetaDataProxy is deprecated. '
+            'Please use core_resources.NsxLibMetadataProxy instead.')
+        super(MetaDataProxy, self).__init__(rest_client)
 
 
-class DhcpProfile(AbstractRESTResource):
-
-    @property
-    def uri_segment(self):
-        return 'dhcp/server-profiles'
-
-    def create(self, *args, **kwargs):
-        pass
-
-    def update(self, uuid, *args, **kwargs):
-        pass
+class DhcpProfile(core_resources.NsxLibDhcpProfile):
+    # TODO(asarfaty): keeping this for backwards compatibility.
+    # This code will be removed in the future.
+    def __init__(self, rest_client, *args, **kwargs):
+        versionutils.report_deprecated_feature(
+            LOG,
+            'resources.DhcpProfile is deprecated. '
+            'Please use core_resources.NsxLibDhcpProfile instead.')
+        super(DhcpProfile, self).__init__(rest_client)
 
 
-class LogicalDhcpServer(AbstractRESTResource):
+class LogicalDhcpServer(utils.NsxLibApiBase):
 
     def get_dhcp_opt_code(self, name):
         _supported_options = {
@@ -546,6 +394,7 @@ class LogicalDhcpServer(AbstractRESTResource):
             'dhcp-client-identifier': 61,
             'nisplus-domain': 64,
             'nisplus-servers': 65,
+            'tftp-server': 66,
             'tftp-server-name': 66,
             'bootfile-name': 67,
             'system-architecture': 93,
@@ -567,6 +416,10 @@ class LogicalDhcpServer(AbstractRESTResource):
     def uri_segment(self):
         return 'dhcp/servers'
 
+    @property
+    def resource_type(self):
+        return 'LogicalDhcpServer'
+
     def _construct_server(self, body, dhcp_profile_id=None, server_ip=None,
                           name=None, dns_nameservers=None, domain_name=None,
                           gateway_ip=False, options=None, tags=None):
@@ -596,23 +449,16 @@ class LogicalDhcpServer(AbstractRESTResource):
         self._construct_server(body, dhcp_profile_id, server_ip, name,
                                dns_nameservers, domain_name, gateway_ip,
                                options, tags)
-        return self._client.create(body=body)
+        return self.client.create(self.get_path(), body=body)
 
     def update(self, uuid, dhcp_profile_id=None, server_ip=None, name=None,
                dns_nameservers=None, domain_name=None, gateway_ip=False,
                options=None, tags=None):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def _do_update():
-            body = self._client.get(uuid)
-            self._construct_server(body, dhcp_profile_id, server_ip, name,
-                                   dns_nameservers, domain_name, gateway_ip,
-                                   options, tags)
-            return self._client.update(uuid, body=body)
-
-        return _do_update()
+        body = {'ipv4_dhcp_server': {}}
+        self._construct_server(body, dhcp_profile_id, server_ip, name,
+                               dns_nameservers, domain_name, gateway_ip,
+                               options, tags)
+        return self._update_with_retry(uuid, body)
 
     def create_binding(self, server_uuid, mac, ip, hostname=None,
                        lease_time=None, options=None, gateway_ip=False):
@@ -627,36 +473,32 @@ class LogicalDhcpServer(AbstractRESTResource):
             # Note that None is valid for gateway_ip, means deleting it.
             body['gateway_ip'] = gateway_ip
         url = "%s/static-bindings" % server_uuid
-        return self._client.url_post(url, body)
+        return self.client.url_post(self.get_path(url), body)
 
     def get_binding(self, server_uuid, binding_uuid):
         url = "%s/static-bindings/%s" % (server_uuid, binding_uuid)
-        return self._client.url_get(url)
+        return self.get(url)
 
     def update_binding(self, server_uuid, binding_uuid, **kwargs):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self._client.max_attempts)
-        def _do_update():
-            body = self.get_binding(server_uuid, binding_uuid)
-            body.update(kwargs)
-            url = "%s/static-bindings/%s" % (server_uuid, binding_uuid)
-            return self._client.url_put(url, body)
-
-        return _do_update()
+        body = {}
+        body.update(kwargs)
+        url = "%s/static-bindings/%s" % (server_uuid, binding_uuid)
+        self._update_resource(self.get_path(url), body, retry=True)
 
     def delete_binding(self, server_uuid, binding_uuid):
         url = "%s/static-bindings/%s" % (server_uuid, binding_uuid)
-        return self._client.url_delete(url)
+        return self.delete(url)
 
 
-class IpPool(AbstractRESTResource):
-    # TODO(asarfaty): Check the DK api - could be different
+class IpPool(utils.NsxLibApiBase):
     @property
     def uri_segment(self):
         return 'pools/ip-pools'
 
+    @property
+    def resource_type(self):
+        return 'IpPool'
+
     def _generate_ranges(self, cidr, gateway_ip):
         """Create list of ranges from the given cidr.
 
@@ -706,11 +548,7 @@ class IpPool(AbstractRESTResource):
         if tags:
             body['tags'] = tags
 
-        return self._client.create(body=body)
-
-    def delete(self, pool_id):
-        """Delete an IPPool by its ID."""
-        return self._client.delete(pool_id)
+        return self.client.create(self.get_path(), body=body)
 
     def _update_param_in_pool(self, args_dict, key, pool_data):
         # update the arg only if it exists in the args dictionary
@@ -741,10 +579,7 @@ class IpPool(AbstractRESTResource):
                                    pool["subnets"][0])
         self._update_param_in_pool(kwargs, 'cidr',
                                    pool["subnets"][0])
-        return self._client.update(pool_id, pool)
-
-    def get(self, pool_id):
-        return self._client.get(pool_id)
+        return self.client.update(self.get_path(pool_id), pool)
 
     def allocate(self, pool_id, ip_addr=None, display_name=None, tags=None):
         """Allocate an IP from a pool."""
@@ -757,15 +592,83 @@ class IpPool(AbstractRESTResource):
             body['tags'] = tags
         if display_name is not None:
             body['display_name'] = display_name
-        return self._client.url_post(url, body=body)
+        return self.client.url_post(self.get_path(url), body=body)
 
     def release(self, pool_id, ip_addr):
         """Release an IP back to a pool."""
         url = "%s?action=RELEASE" % pool_id
         body = {"allocation_id": ip_addr}
-        return self._client.url_post(url, body=body)
+        return self.client.url_post(self.get_path(url), body=body)
 
     def get_allocations(self, pool_id):
         """Return information about the allocated IPs in the pool."""
         url = "%s/allocations" % pool_id
-        return self._client.url_get(url)
+        return self.client.url_get(self.get_path(url))
+
+
+class NodeHttpServiceProperties(utils.NsxLibApiBase):
+    @property
+    def uri_segment(self):
+        return 'node/services/http'
+
+    @property
+    def resource_type(self):
+        return 'NodeHttpServiceProperties'
+
+    def get_properties(self):
+        return self.client.get(self.get_path())
+
+    def get_rate_limit(self):
+        if (self.nsxlib and
+            not self.nsxlib.feature_supported(
+                nsx_constants.FEATURE_RATE_LIMIT)):
+            msg = (_("Rate limit is not supported by NSX version %s") %
+                   self.nsxlib.get_version())
+            raise exceptions.ManagerError(details=msg)
+
+        properties = self.get_properties()
+        return properties.get('service_properties', {}).get(
+            'client_api_rate_limit')
+
+    def update_rate_limit(self, value):
+        """update the NSX rate limit. default value is 40. 0 means no limit"""
+        if (self.nsxlib and
+            not self.nsxlib.feature_supported(
+                nsx_constants.FEATURE_RATE_LIMIT)):
+            msg = (_("Rate limit is not supported by NSX version %s") %
+                   self.nsxlib.get_version())
+            raise exceptions.ManagerError(details=msg)
+
+        properties = self.get_properties()
+        if 'service_properties' in properties:
+            properties['service_properties'][
+                'client_api_rate_limit'] = int(value)
+
+        # update the value using a PUT command, which is expected to return 202
+        expected_results = [requests.codes.accepted]
+        self.client.update(self.uri_segment, properties,
+                           expected_results=expected_results)
+
+        # restart the http service using POST, which is expected to return 202
+        restart_url = self.uri_segment + '?action=restart'
+        self.client.create(restart_url, expected_results=expected_results)
+
+    def delete(self, uuid):
+        """Not supported"""
+        msg = _("Delete is not supported for %s") % self.uri_segment
+        raise exceptions.ManagerError(details=msg)
+
+    def get(self, uuid):
+        """Not supported"""
+        msg = _("Get is not supported for %s") % self.uri_segment
+        raise exceptions.ManagerError(details=msg)
+
+    def list(self):
+        """Not supported"""
+        msg = _("List is not supported for %s") % self.uri_segment
+        raise exceptions.ManagerError(details=msg)
+
+    def find_by_display_name(self, display_name):
+        """Not supported"""
+        msg = _("Find is not supported for %s") % self.uri_segment
+        raise exceptions.ManagerError(details=msg)
