@@ -18,11 +18,10 @@
 NSX-V3 Plugin security & Distributed Firewall integration module
 """
 
-from neutron_lib import constants
 from oslo_log import log
 from oslo_utils import excutils
 
-from vmware_nsxlib._i18n import _LE, _LW
+from vmware_nsxlib.v3 import constants
 from vmware_nsxlib.v3 import exceptions
 from vmware_nsxlib.v3 import nsx_constants as consts
 from vmware_nsxlib.v3 import utils
@@ -36,9 +35,19 @@ MAX_NSGROUPS_CRITERIA_TAGS = 10
 
 class NsxLibNsGroup(utils.NsxLibApiBase):
 
-    def __init__(self, client, max_attempts, firewall_section_handler):
+    def __init__(self, client, nsxlib_config, firewall_section_handler,
+                 nsxlib=None):
         self.firewall_section = firewall_section_handler
-        super(NsxLibNsGroup, self).__init__(client, max_attempts)
+        super(NsxLibNsGroup, self).__init__(client, nsxlib_config,
+                                            nsxlib=nsxlib)
+
+    @property
+    def uri_segment(self):
+        return 'ns-groups'
+
+    @property
+    def resource_type(self):
+        return 'NSGroup'
 
     def update_on_backend(self, context, security_group,
                           nsgroup_id, section_id,
@@ -89,7 +98,7 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
                     sg_id=nsgroup_id)
             except exceptions.ResourceNotFound:
                 with excutils.save_and_reraise_exception():
-                    LOG.error(_LE("NSGroup %s doesn't exists"), nsgroup_id)
+                    LOG.error("NSGroup %s doesn't exists", nsgroup_id)
         for nsgroup_id in removed:
             self.remove_member(
                 nsgroup_id, consts.TARGET_TYPE_LOGICAL_PORT, lport_id)
@@ -99,6 +108,10 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
         service.update(properties)
         return {'service': service}
 
+    def get_nsgroup_complex_expression(self, expressions):
+        return {'resource_type': consts.NSGROUP_COMPLEX_EXP,
+                'expressions': expressions}
+
     def get_switch_tag_expression(self, scope, tag):
         return {'resource_type': consts.NSGROUP_TAG_EXP,
                 'target_type': consts.TARGET_TYPE_LOGICAL_SWITCH,
@@ -112,11 +125,11 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
                 'tag': tag}
 
     def create(self, display_name, description, tags,
-               membership_criteria=None):
+               membership_criteria=None, members=None):
         body = {'display_name': display_name,
                 'description': description,
                 'tags': tags,
-                'members': []}
+                'members': [] if members is None else members}
         if membership_criteria:
             # Allow caller to pass a list of membership criterias.
             # The 'else' block is maintained for backwards compatibility
@@ -125,32 +138,33 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
                 body.update({'membership_criteria': membership_criteria})
             else:
                 body.update({'membership_criteria': [membership_criteria]})
-        return self.client.create('ns-groups', body)
+        return self.client.create(self.get_path(), body)
 
     def list(self):
         return self.client.list(
-            'ns-groups?populate_references=false').get('results', [])
+            '%s?populate_references=false' % self.get_path()).get(
+            'results', [])
 
     def update(self, nsgroup_id, display_name=None, description=None,
-               membership_criteria=None, members=None):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self.nsxlib_config.max_attempts)
-        def _do_update():
-            nsgroup = self.read(nsgroup_id)
-            if display_name is not None:
-                nsgroup['display_name'] = display_name
-            if description is not None:
-                nsgroup['description'] = description
-            if members is not None:
-                nsgroup['members'] = members
-            if membership_criteria is not None:
+               membership_criteria=None, members=None, tags_update=None):
+        nsgroup = {}
+        if display_name is not None:
+            nsgroup['display_name'] = display_name
+        if description is not None:
+            nsgroup['description'] = description
+        if members is not None:
+            nsgroup['members'] = members
+        if membership_criteria is not None:
+            if isinstance(membership_criteria, list):
+                nsgroup['membership_criteria'] = membership_criteria
+            else:
                 nsgroup['membership_criteria'] = [membership_criteria]
-            return self.client.update(
-                'ns-groups/%s' % nsgroup_id, nsgroup)
-
-        return _do_update()
+        if tags_update is not None:
+            nsgroup['tags_update'] = tags_update
+        return self._update_resource(
+            self.get_path(nsgroup_id), nsgroup,
+            get_params='?populate_references=true',
+            retry=True)
 
     def get_member_expression(self, target_type, target_id):
         return {
@@ -161,7 +175,7 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
             'value': target_id}
 
     def _update_with_members(self, nsgroup_id, members, action):
-        members_update = 'ns-groups/%s?action=%s' % (nsgroup_id, action)
+        members_update = '%s?action=%s' % (self.get_path(nsgroup_id), action)
         return self.client.create(members_update, members)
 
     def add_members(self, nsgroup_id, target_type, target_ids):
@@ -179,8 +193,8 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
         except exceptions.ManagerError:
             # REVISIT(roeyc): A ManagerError might have been raised for a
             # different reason, e.g - NSGroup does not exists.
-            LOG.warning(_LW("Failed to add %(target_type)s resources "
-                            "(%(target_ids)s) to NSGroup %(nsgroup_id)s"),
+            LOG.warning("Failed to add %(target_type)s resources "
+                        "(%(target_ids)s) to NSGroup %(nsgroup_id)s",
                         {'target_type': target_type,
                          'target_ids': target_ids,
                          'nsgroup_id': nsgroup_id})
@@ -202,12 +216,12 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
 
     def read(self, nsgroup_id):
         return self.client.get(
-            'ns-groups/%s?populate_references=true' % nsgroup_id)
+            '%s?populate_references=true' % self.get_path(nsgroup_id))
 
     def delete(self, nsgroup_id):
         try:
             return self.client.delete(
-                'ns-groups/%s?force=true' % nsgroup_id)
+                '%s?force=true' % self.get_path(nsgroup_id))
         # FIXME(roeyc): Should only except NotFound error.
         except Exception:
             LOG.debug("NSGroup %s does not exists for delete request.",
@@ -223,16 +237,27 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
 
 class NsxLibFirewallSection(utils.NsxLibApiBase):
 
+    @property
+    def uri_segment(self):
+        return 'firewall/sections'
+
+    @property
+    def resource_type(self):
+        return 'FirewallSection'
+
     def add_member_to_fw_exclude_list(self, target_id, target_type):
         resource = 'firewall/excludelist?action=add_member'
         body = {"target_id": target_id,
                 "target_type": target_type}
-        self.client.create(resource, body)
+        self._create_with_retry(resource, body)
 
     def remove_member_from_fw_exclude_list(self, target_id, target_type):
         resource = ('firewall/excludelist?action=remove_member&object_id='
                     + target_id)
-        self.client.create(resource)
+        self._create_with_retry(resource)
+
+    def get_excludelist(self):
+        return self.client.list('firewall/excludelist')
 
     def _get_direction(self, sg_rule):
         return (
@@ -309,73 +334,77 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
                      applied_tos, tags,
                      operation=consts.FW_INSERT_BOTTOM,
                      other_section=None):
-        resource = 'firewall/sections?operation=%s' % operation
+        resource = '%s?operation=%s' % (self.uri_segment, operation)
         body = self._build(display_name, description,
                            applied_tos, tags)
         if other_section:
             resource += '&id=%s' % other_section
-        return self.client.create(resource, body)
+        return self._create_with_retry(resource, body)
 
     def create_with_rules(self, display_name, description, applied_tos=None,
                           tags=None, operation=consts.FW_INSERT_BOTTOM,
                           other_section=None, rules=None):
-        resource = 'firewall/sections?operation=%s' % operation
+        resource = '%s?operation=%s' % (self.uri_segment, operation)
         body = {
             'display_name': display_name,
             'description': description,
             'stateful': True,
             'section_type': consts.FW_SECTION_LAYER3,
             'applied_tos': applied_tos or [],
-            'rules': rules or [],
             'tags': tags or []
         }
-        if rules:
+        if rules is not None:
             resource += '&action=create_with_rules'
+            body['rules'] = rules
         if other_section:
             resource += '&id=%s' % other_section
-        return self.client.create(resource, body)
+        return self._create_with_retry(resource, body)
 
     def update(self, section_id, display_name=None, description=None,
-               applied_tos=None, rules=None, tags_update=None):
-        # Using internal method so we can access max_attempts in the decorator
-        @utils.retry_upon_exception(
-            exceptions.StaleRevision,
-            max_attempts=self.nsxlib_config.max_attempts)
-        def _do_update():
-            resource = 'firewall/sections/%s' % section_id
-            section = self.read(section_id)
+               applied_tos=None, rules=None, tags_update=None, force=False):
+        resource = self.get_path(section_id)
+        params = None
+        section = {}
+        if rules is not None:
+            params = '?action=update_with_rules'
+            section['rules'] = rules
+        if display_name is not None:
+            section['display_name'] = display_name
+        if description is not None:
+            section['description'] = description
+        if applied_tos is not None:
+            section['applied_tos'] = [self.get_nsgroup_reference(nsg_id)
+                                      for nsg_id in applied_tos]
+        if tags_update is not None:
+            section['tags_update'] = tags_update
 
-            if rules is not None:
-                resource += '?action=update_with_rules'
-                section.update({'rules': rules})
-            if display_name is not None:
-                section['display_name'] = display_name
-            if description is not None:
-                section['description'] = description
-            if applied_tos is not None:
-                section['applied_tos'] = [self.get_nsgroup_reference(nsg_id)
-                                          for nsg_id in applied_tos]
-            if tags_update is not None:
-                section['tags'] = utils.update_v3_tags(section.get('tags', []),
-                                                       tags_update)
-            if rules is not None:
-                return self.client.create(resource, section)
-            elif any(p is not None for p in (display_name, description,
-                                             applied_tos)):
-                return self.client.update(resource, section)
+        headers = None
+        if force:
+            # shared sections (like default section) can serve multiple
+            # openstack deployments. If some operate under protected
+            # identities, force-overwrite is needed.
+            # REVISIT(annak): find better solution for shared sections
+            headers = {'X-Allow-Overwrite': 'true'}
 
-        return _do_update()
+        if rules is not None:
+            return self._update_resource(resource, section,
+                                         headers=headers,
+                                         create_action=True,
+                                         action_params=params,
+                                         retry=True)
 
-    def read(self, section_id):
-        resource = 'firewall/sections/%s' % section_id
-        return self.client.get(resource)
+        elif any(p is not None for p in (display_name, description,
+                                         applied_tos, tags_update)):
+            return self._update_resource(resource, section,
+                                         headers=headers,
+                                         action_params=params,
+                                         retry=True)
 
     def list(self):
-        resource = 'firewall/sections'
-        return self.client.list(resource).get('results', [])
+        return self.client.list(self.get_path()).get('results', [])
 
     def delete(self, section_id):
-        resource = 'firewall/sections/%s?cascade=true' % section_id
+        resource = '%s?cascade=true' % self.get_path(section_id)
         return self.client.delete(resource)
 
     def get_nsgroup_reference(self, nsgroup_id):
@@ -427,24 +456,30 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
             rule_dict['applied_tos'] = applied_tos
         return rule_dict
 
-    def add_rule(self, rule, section_id):
-        resource = 'firewall/sections/%s/rules' % section_id
-        params = '?operation=insert_bottom'
-        return self.client.create(resource + params, rule)
+    def add_rule(self, rule, section_id, operation=consts.FW_INSERT_BOTTOM):
+        resource = '%s/rules' % self.get_path(section_id)
+        params = '?operation=%s' % operation
+        return self._create_with_retry(resource + params, rule)
 
-    def add_rules(self, rules, section_id):
-        resource = 'firewall/sections/%s/rules' % section_id
-        params = '?action=create_multiple&operation=insert_bottom'
-        return self.client.create(resource + params, {'rules': rules})
+    def add_rules(self, rules, section_id, operation=consts.FW_INSERT_BOTTOM):
+        resource = '%s/rules' % self.get_path(section_id)
+        params = '?action=create_multiple&operation=%s' % operation
+        return self._create_with_retry(resource + params, {'rules': rules})
 
     def delete_rule(self, section_id, rule_id):
-        resource = 'firewall/sections/%s/rules/%s' % (section_id, rule_id)
-        return self.client.delete(resource)
+        resource = '%s/rules/%s' % (section_id, rule_id)
+        return self._delete_with_retry(resource)
 
     def get_rules(self, section_id):
-        resource = 'firewall/sections/%s/rules' % section_id
+        resource = '%s/rules' % self.get_path(section_id)
         return self.client.get(resource)
 
+    def get_default_rule(self, section_id):
+        rules = self.get_rules(section_id)['results']
+        last_rule = rules[-1]
+        if last_rule['is_default']:
+            return last_rule
+
     def _get_fw_rule_from_sg_rule(self, sg_rule, nsgroup_id, rmt_nsgroup_id,
                                   logged, action):
         # IPV4 or IPV6
@@ -551,5 +586,51 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
                     applied_tos=nested_groups,
                     rules=[dhcp_client_rule_out,
                            dhcp_client_rule_in,
-                           block_rule])
+                           block_rule],
+                    force=True)
         return section['id']
+
+
+class NsxLibIPSet(utils.NsxLibApiBase):
+
+    @property
+    def uri_segment(self):
+        return 'ip-sets'
+
+    @property
+    def resource_type(self):
+        return 'IPSet'
+
+    def create(self, display_name, description=None, ip_addresses=None,
+               tags=None):
+        body = {
+            'display_name': display_name,
+            'description': description or '',
+            'ip_addresses': ip_addresses or [],
+            'tags': tags or []
+        }
+        return self.client.create(self.get_path(), body)
+
+    def update(self, ip_set_id, display_name=None, description=None,
+               ip_addresses=None, tags_update=None):
+        ip_set = {}
+        if tags_update:
+            ip_set['tags_update'] = tags_update
+        if display_name is not None:
+            ip_set['display_name'] = display_name
+        if description is not None:
+            ip_set['description'] = description
+        if ip_addresses is not None:
+            ip_set['ip_addresses'] = ip_addresses
+        return self._update_resource(self.get_path(ip_set_id),
+                                     ip_set, retry=True)
+
+    def read(self, ip_set_id):
+        return self.client.get('ip-sets/%s' % ip_set_id)
+
+    def delete(self, ip_set_id):
+        self._delete_with_retry(ip_set_id)
+
+    def get_ipset_reference(self, ip_set_id):
+        return {'target_id': ip_set_id,
+                'target_type': consts.IP_SET}
